#! /usr/bin/python

## pd.py
## Copyright (C) 2007 Ivan Uemlianin
## Contact: ivan@llaisdy.com
## 
## This file is part of Trefnydd.
##
## Trefnydd is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Trefnydd is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Trefnydd (i.e., doc/gpl.txt); if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
## or see http://www.gnu.org/licenses/gpl.txt


"""
========
trefnydd
========

Package:   trefnydd
Module:    pd
Copyright: 2007 Ivan A. Uemlianin
Contact:   ivan@llaisdy.com
Licence:   GNU GPL (see doc/gpl.txt)

pd
==

"""


import os
import copy, pprint
from trefnydd.utils import file_IO, location
from trefnydd.lib import phones, g2p

    
class PDEntry(object):
    def __init__(self, orth=''):
        self.graphemeSet = None  # these three would generally
        self.phoneSet = None     # be set to point to those of
        self.g2pRuleSet = None   # the container (eg PD)
        self.localRules = None # g2p ruleset 
        self.orthography = orth
        self.graphemeString = None
        self.gStr_exceptional = False
        self.phoneString = None
        self.g2pRulesFired = [] # ordered list of (pointers to) g2p rules

        self.comments = []
        # list of user entered comments, eg on altered g or g2p.

        self.ptOfSpeechSet = None
        self.ptOfSpeech = None
        
    def __str__(self):
        out = '%s\n\t(gr: %s)\n\t(ph: %s)\n' % (self.orthography,
                                           self.graphemeString.__str__(),
                                           self.phoneString.__str__())
        if self.comments:
            out = '%s\n\t%s\n\n' % (out, '\n\t'.join([c for c in self.comments]))
        
        return out

    def getName(name):
        return self.orthography

    name = property(getName)

    def pStr_isExceptional(self):
        return len(self.localRules) > 0

    def isResolved(self):
        return self.phoneString.isinstance(phones.PhoneString)

    def applyRules(self):
        self.graphemeString = GraphemeString()
        self.graphemeString.setGraphemeSet(GRAPHEME_SET_CYMRAEG)
        self.graphemeString.importString(self.orthography)
        # todo: print self.graphemeString.rulesFired # ?
        ps, rules = self.g2pRuleSet.transform(self.graphemeString, True)
        self.set_phoneString(ps)
        self.g2pRulesFired = rules

    def set_orthography(self, orthography):
        self.orthography = orthography

    def get_orthography(self):
        return self.orthography
    
    def set_graphemeString(self, graphemeString):
        self.graphemeString = graphemeString

    def get_graphemeString(self):
        return self.graphemeString
    
    def set_phoneString(self, phoneString):
        self.phoneString = phoneString

    def get_phoneString(self):
        return self.phoneString
    
    def add_comment(self, comment):
        self.comments.append(comment)

    def remove_comment(self, comment):
        self.comments.remove(comment)

    def get_comments(self):
        return self.comments

    def fits(self, template):
        """
        template is an underspecified pdEntry
        returns True if self fits template
        """
        pass

    
class PronunciationDictionary(location.Located):
    # todo: include split label dialog
    # set better than dict - can sort by different features
    extPointer = {'.graphemeset': 'Grapheme Set',
                  '.phoneset': 'Phone Set',
                  '.g2p': 'G2P Rules',
                  '.pde': 'Lexicon'}
    def __init__(self, debug=False):
        location.Located.__init__(self)
        self.debug = debug
        self.name = 'pd'
        self.graphemeSet = g2p.GraphemeSet()
        self.phoneSet = phones.Phoneset()
        self.g2pRuleSet = g2p.G2PRuleSet()
        self.entries = set()

    def getFacets(self):
        """
        For display in treeCtrl:
        audio, tiers
        """
        facets = {}
        facets['Audio'] = self.audioData
        for t in [self.graphemeSet, self.phoneSet,
                  self.g2pRuleSet, self.entries]:
            facets[t.name] = t
        return facets

    facets = property(getFacets)

    def __str__(self):
        out = '* Pronunciation Dictionary: %s\n\n' % self.name
        # todo
        # sort order!
        wl = self.getEntriesBySortOrder()
        for pde in wl:
            out = '%s%s\n' % (out, pde.__str__())
        out = '%s\n\n\n------------------------------\n' % out
        return out
        
    # todo: search and diagnostic methods
    #       (eg get incomplete entries)

    def save(self):
        if self.debug:
            print 'pd.save()'
        for item in [self.graphemeSet,
                     self.phoneSet,
                     self.g2pRuleSet]:
            item.homeDir = self.homeDir
            item.save()
        self.saveEntries()

    def load(self):
        if self.debug: print 'pd.load()'
        for ftup in [(self.extPointer[os.path.splitext(f)[1]], f)
                     for f in os.listdir(self.homeDir)
                     if os.path.splitext(f)[1] in self.extPointer.keys()]:
            if self.debug:
                print '\t%s: %s'% (ftup[0], ftup[1])
            fullfn = os.path.join(self.homeDir, ftup[1])
            self.importFromFile(ftup[0], fullfn)

    def saveEntries(self):
        entries = self.getEntryList()
        fn = os.path.join(self.homeDir, 'pd.pde')
        open(fn, 'w').write(entries)

    def getEntryList(self):
        out = ''
        for e in self.entries:
            out = '%s%s\n' % (out, e.__str__())
        return out

    def set_graphemeSet(self, graphemeSet):
        self.graphemeSet = graphemeSet

    def get_graphemeSet(self):
        return self.graphemeSet
    
    def set_phoneSet(self, phoneSet):
        self.phoneSet = phoneSet

    def get_phoneSet(self):
        return self.phoneSet
    
    def set_g2pRuleSet(self, g2pRuleSet):
        self.g2pRuleSet = g2pRuleSet

    def get_graphemeSet(self):
        return self.graphemeSet

    def importFromFile(self, thing, fn):
        """thing in 'Grapheme Set', 'Phone Set', 'G2P Rules', 'Lexicon'
        """
        if thing == 'Grapheme Set':
            self.graphemeSet.importDataFromFile(fn)
            self.g2pRuleSet.setGraphemeSet(self.graphemeSet)
        elif thing == 'Phone Set':
            self.phoneSet.importDataFromFile(fn)
            self.g2pRuleSet.setPhoneSet(self.phoneSet)
        elif thing == 'G2P Rules':
            self.g2pRuleSet.setGraphemeSet(self.graphemeSet)
            self.g2pRuleSet.setPhoneSet(self.phoneSet)
            self.g2pRuleSet.importDataFromFile(fn) #, False)
        elif thing == 'Lexicon':
            # todo
            pass
            
    
    def addEntry(self, pdEntry):
        self.entries.add(pdEntry)

    def importWordList(self, wlist):
        for w in wlist:
            self.initialiseEntryOrth(w)

    def getEntries(self, template):
        # todo
        # template is an underspecified PDEntry
        # getEntries returns a set of pdEntries which fit template
        subset = set()
        for entry in self.entries:
            if entry.fits(template):
                subset.add(entry)
        return subset

    def getEntriesBySortOrder(self, order='graphemeString'):
        """
        Sort order = graphemeString (default) | orthography | phoneString
        """
        wl = [(e.__getattribute__(order), e) for e in self.entries]
        wl.sort()
        wl.reverse()
        wl = [i[1] for i in wl]
        return wl
        
    def initialiseEntryOrth(self, string):
        # todo: doesn't test if entry is already in self
        pde = PDEntry()
        pde.set_orthography(string)
        gs = g2p.GraphemeString()
        gs.setGraphemeSet(self.graphemeSet)
        gs.importString(string)
        pde.set_graphemeString(gs)
        pde.set_phoneString(self.g2pRuleSet.transform(gs))
        self.addEntry(pde)
        
    def initialiseEntryPhon(self, pString):
        pass

    def applyRules(self):
        for pde in self.entries:
            pde.applyRules()


    def importPD(self, fn, format=''):
        if format == 'festival':
            self.importFestivalPD(fn)
        else:
            pass

    def importFestivalPD(fn):
        # todo
        pass
    

rhestrGeir = ['angharad',
              'llanddeiniolen',
              'llanfairpwllgwyngyll',
              'llangollen',
              'llefrith',
              'llyfrgell',
              'llyfrgellydd'
              ]

def main():
    pd = PronunciationDictionary()
##     pd.set_graphemeSet(g2p.GRAPHEME_SET_CYMRAEG)
##     pd.set_phoneSet(phones.PHONESET_IPA)
##     pd.set_g2pRuleSet(g2p.G2PRULESET_CYMRAEG)

    pd.importWordList(rhestrGeir)
    
    print pd
    
    
if __name__ == '__main__':
    main()

